@@ -1,5 +1,5 @@
 package VM::JiffyBox;
-$VM::JiffyBox::VERSION = '0.027'; # TRIAL
+$VM::JiffyBox::VERSION = '0.03'; # TRIAL
 # The line below is recognised by Dist::Zilla and taken for CPAN packaging
 # ABSTRACT: OO-API for JiffyBox Virtual Machine
 
@@ -125,11 +125,45 @@ sub create_vm {
     }
 
     my $box_id = $self->last->{result}->{id};
-    my $box = VM::JiffyBox::Box->new(id => $box_id, hypervisor => $self);
+    my $box = VM::JiffyBox::Box->new(id => $box_id, hypervisor => $self, name => $args->{name});
 
     return $box;
 }
 
+sub get_vms {
+    my ($self) = shift;
+
+    my $url      = $self->base_url . '/jiffyBoxes';
+    my $response = $self->ua->get( $url );
+
+    unless ( $response->is_success ) {
+        $self->last ($response->status_line);
+        return;
+    }
+
+    $self->last( from_json( $response->decoded_content ) );
+
+    if (exists $self->last->{messages}->[0]->{type}
+        and    $self->last->{messages}->[0]->{type} eq 'error') {
+        return;
+    }
+
+    my @boxes;
+
+    my $result = $self->last->{result};
+    for my $box_id ( keys %{ $result || {} } ) {
+        my $name = $result->{$box_id}->{name};
+
+        push @boxes, VM::JiffyBox::Box->new(
+            id         => $box_id,
+            name       => $name,
+            hypervisor => $self,
+        );
+    }
+
+    return @boxes;
+}
+
 sub get_distributions {
     my ($self) = shift;
 
@@ -192,7 +226,7 @@ VM::JiffyBox - OO-API for JiffyBox Virtual Machine
 
 =head1 VERSION
 
-version 0.027
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -350,6 +384,16 @@ Returns details for a plan-id or name (pricing model).
 Pass name or ID as an argument to the method.
 Returns a hashref.
 
+=head2 get_vms
+
+Returns a list of C<VM::JiffyBox::Box> objects, where each object
+represents an existing box.
+
+  my @boxes = $jb->get_vms();
+  for my $box ( @boxes ) {
+      print $box->name, "\n";
+  }
+
 =head1 METHODS (SHORTCUTS)
 
 Methods which are not part of the official API, but provide some often needed calls by using the API mentioned above.
