@@ -1,5 +1,5 @@
 package VM::JiffyBox::Box;
-$VM::JiffyBox::Box::VERSION = '0.027'; # TRIAL
+$VM::JiffyBox::Box::VERSION = '0.03'; # TRIAL
 # ABSTRACT: Representation of a Virtual Machine in JiffyBox
 
 use strict;
@@ -8,8 +8,9 @@ use warnings;
 use Moo;
 use JSON;
 
-has id         => (is => 'rw', required => 1);
-has hypervisor => (is => 'rw', required => 1);
+has id         => (is => 'ro', required => 1);
+has hypervisor => (is => 'ro', required => 1);
+has name       => (is => 'rwp');
 
 has last          => (is => 'rw');
 has backup_cache  => (is => 'rw');
@@ -17,6 +18,8 @@ has details_cache => (is => 'rw');
 has start_cache   => (is => 'rw');
 has stop_cache    => (is => 'rw');
 has delete_cache  => (is => 'rw');
+has freeze_cache  => (is => 'rw');
+has thaw_cache    => (is => 'rw');
 
 sub get_backups {
     my $self = shift;
@@ -70,11 +73,55 @@ sub get_details {
     return                $details ;
 }
 
-sub start {
+sub clone {
     my $self = shift;
+    my $args = shift;
+    
+    # POSSIBLE EXIT (DIE)
+    die 'name needed'                     unless $args->{name};
+    die 'planid needed'                   unless $args->{planid};
+
+    my $url  = $self->{hypervisor}->base_url . '/jiffyBoxes/' . $self->id;
+    my $json = to_json( {
+        name     => $args->{name},
+        planid   => $args->{planid},
+        metadata => $args->{metadata} || {},
+    });
+
+    # POSSIBLE EXIT
+    return { url => $url, json => $json }
+        if ($self->{hypervisor}->test_mode);
+    
+    # send the request with method specific json content
+    my $response = $self->{hypervisor}->ua->post( $url, Content => $json ); 
+
+    # POSSIBLE EXIT
+    unless ($response->is_success) {
+
+        $self->last ( $response->status_line );
+        return;
+    }
+
+    my $clone_info = from_json($response->decoded_content);
+
+    my $clone_box = __PACKAGE__->new(
+        id         => $clone_info->{result}->{id},
+        name       => $args->{name},
+        hypervisor => $self,
+    );
+
+    return $clone_box;
+}
+
+sub _status_action {
+    my $self   = shift;
+    my $action = shift;
+
+    my $status = uc $action;
+    $status    = 'SHUTDOWN' if 'stop' eq lc $action;
     
     my $url  = $self->{hypervisor}->base_url . '/jiffyBoxes/' . $self->id;
-    my $json = to_json( { status => 'START' } );
+    my $json = to_json( { status => $status } );
     
     # POSSIBLE EXIT
     return { url => $url, json => $json }
@@ -90,37 +137,36 @@ sub start {
         return;
     }
 
-    my $start_info = from_json($response->decoded_content);
+    my $status_info = from_json($response->decoded_content);
 
-    $self->last        ($start_info);
-    $self->start_cache ($start_info);
-    return              $start_info ;
+    my $cache_sub = $self->can( $action . '_cache' );
+    $self->last ($status_info);
+    $cache_sub->($status_info) if $cache_sub;
+    return       $status_info ;
+}
+
+sub freeze {
+    my $self = shift;
+
+    return $self->_status_action( 'freeze' );
+}
+
+sub thaw {
+    my $self = shift;
+
+    return $self->_status_action( 'thaw' );
+}
+
+sub start {
+    my $self = shift;
+
+    return $self->_status_action( 'start' );
 }
 
 sub stop {
     my $self = shift;
-    
-    my $url  = $self->{hypervisor}->base_url . '/jiffyBoxes/' . $self->id;
-    my $json = to_json( { status => 'SHUTDOWN' } );
-    
-    # POSSIBLE EXIT
-    return { url => $url, json => $json }
-        if ($self->{hypervisor}->test_mode);
-    
-    my $response = $self->{hypervisor}->ua->put( $url, Content => $json ); 
 
-    # POSSIBLE EXIT
-    unless ($response->is_success) {
-
-        $self->last ( $response->status_line );
-        return;
-    }
-
-    my $stop_info = from_json($response->decoded_content);
-
-    $self->last       ($stop_info);
-    $self->stop_cache ($stop_info);
-    return             $stop_info ;
+    return $self->_status_action( 'stop' );
 }
 
 sub delete {
@@ -142,9 +188,9 @@ sub delete {
 
     my $delete_info = from_json($response->decoded_content);
 
-    $self->last       ($delete_info);
-    $self->stop_cache ($delete_info);
-    return             $delete_info ;
+    $self->last         ($delete_info);
+    $self->delete_cache ($delete_info);
+    return               $delete_info ;
 }
 
 1;
@@ -161,7 +207,7 @@ VM::JiffyBox::Box - Representation of a Virtual Machine in JiffyBox
 
 =head1 VERSION
 
-version 0.027
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -233,6 +279,14 @@ Contains information of the last call to stop().
 
 Contains information of the last call to delete().
 
+=item freeze_cache
+
+Contains information of the last call to freeze().
+
+=item thaw_cache
+
+Contains information of the last call to thaw().
+
 =back
 
 =head1 METHODS
@@ -242,7 +296,7 @@ All methods (exluding C<new>) will return information about the request, instead
 =head2 new
 
 Creates a box-object.
-Requires two paramters.
+Requires two parameters.
 
 =over
 
@@ -259,6 +313,16 @@ Required.
 
 =back
 
+Optional parameters
+
+=over 4
+
+=item * name
+
+Name of the box
+
+=back
+
 =head2 get_details
 
 Returns hashref with information about the virtual machine.
@@ -269,6 +333,15 @@ Takes no arguments.
 Returns hashref with information about the backups of the virtual machine.
 Takes no arguments.
 
+=head2 clone
+
+Clones a virtual machine. Needs a C<name> and a C<planid>. Returns a C<VM::JiffyBox::Box> object.
+
+  my $clone = $box->clone(
+      name   => 'Clonename',
+      planid => 22,
+  );
+
 =head2 start
 
 Starts a virtual machine.
@@ -278,6 +351,20 @@ It must be ensured (by you) that the machine has the state C<READY> before calli
 
 Stop a virtual machine.
 
+  $box->stop();
+
+=head2 freeze
+
+freeze a virtual machine.
+
+  $box->freeze();
+
+=head2 thaw
+
+Thaw a virtual machine.
+
+  $box->thaw();
+
 =head2 delete
 
 Delete a virtual machine.
